/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

package com.ael.compta.actions.graphics;

import java.io.File;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;
import java.awt.Color;
import java.awt.Font;

import net.sf.jasperreports.engine.JasperRunManager;

import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.Session;
import org.experlog.openeas.api.ServletRequest;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.category.DefaultCategoryDataset;


public class SoldeCompte extends Action {

	@Override
	public boolean serverSide() {
		return true;
	}
	
	@Override
	public boolean service(Session session, ServletRequest r, Object info) throws Exception {
		String periodes[] = r.getParameter("periodes").split(",");
		String soldes[] = r.getParameter("soldes").split(",");
		String numeroCompte = r.getParameter("Numero_Compte");
		String nomFichier = r.getParameter("fichierPng");
			
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
						
		for (int i = 0; i < periodes.length; i++) {
			dataset.addValue(Double.parseDouble(soldes[i]), numeroCompte, periodes[i]);
		}
			
		//  Create the chart object
		CategoryAxis xAxis = new CategoryAxis("P\u00E9riode");
		xAxis.setCategoryLabelPositions(CategoryLabelPositions.DOWN_45);
		xAxis.setLowerMargin(0.05);
		xAxis.setUpperMargin(0.05);
		xAxis.setCategoryMargin(0.3);
		
		NumberAxis yAxis = new NumberAxis("CA en euros");
		yAxis.setAutoRangeIncludesZero(false);
		yAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
			
		BarRenderer renderer = new BarRenderer();
		renderer.setMaxBarWidth(0.03);
		DecimalFormat df = new DecimalFormat("0");
		renderer.setItemLabelGenerator(new StandardCategoryItemLabelGenerator("{2}", df));
		renderer.setItemLabelsVisible(true);
		renderer.setBaseItemLabelsVisible(true);
		renderer.setSeriesPaint(0, new Color(72, 209, 204));
		
		CategoryPlot plot = new CategoryPlot(dataset, xAxis, yAxis, renderer);
			
		JFreeChart chart = new JFreeChart("", JFreeChart.DEFAULT_TITLE_FONT, plot, true);
			
		chart.setBackgroundPaint(new Color(255,255,255,0));
			
		LegendTitle legende = chart.getLegend();
		chart.clearSubtitles() ;
		Font old_police = legende.getItemFont();
		legende.setItemFont(new Font(old_police.getFontName(), old_police.getStyle(), 10));
		chart.addSubtitle(legende);
			
		try {
			String fichierPng = session.getAppConfiguration().get("opensi.absdir") + "/graph/" + nomFichier;
			File file = new File(fichierPng);

			ChartUtilities.saveChartAsPNG(file, chart, 550, 400, null, true, 9);

			file.deleteOnExit();
			
			if (r.getParameter("sortie").equals("pdf"))
				genererPdf(session, r, fichierPng);
				
				
		} catch(SecurityException se) {
			session.trace("Impossible d'enregistrer le fichier du graphique");
			se.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return true;
	}
	
	private void genererPdf(Session s, ServletRequest r, String fichierGraphique) throws Exception {
		String absdir = s.getAppConfiguration().get("opensi.absdir");
		String dossierId = s.getCookie().get("Dossier_Id");
		String fichierPdf = absdir +"/pdf/" + r.getParameter("fichierPdf");
    	String fichierJasper = absdir + "/jasper/" + dossierId + "/Soldes.jasper";
    	
    	Map<String,Object> parametres = new HashMap<String,Object>();
		parametres.put("graphique", fichierGraphique);
		parametres.put("dossierId", dossierId);
		parametres.put("raisonSociale", r.getParameter("raisonSociale"));
		parametres.put("exercice", r.getParameter("exercice"));
		JasperRunManager.runReportToPdfFile(fichierJasper, fichierPdf, parametres);
	}
	
}
